(function() {
  var CouchFunction, DesignDoc, MissingPropPathError, NotAFunctionError, createSandbox, findModule, inspect, makeInitialRefStack, makeRequireFun, puts, readPath, runInNewContext, _ref;
  var __slice = Array.prototype.slice, __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
    for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
    function ctor() { this.constructor = child; }
    ctor.prototype = parent.prototype;
    child.prototype = new ctor;
    child.__super__ = parent.prototype;
    return child;
  };
  runInNewContext = process.binding('evals').Script.runInNewContext;
  _ref = require('sys'), puts = _ref.puts, inspect = _ref.inspect;
  exports.DesignDoc = DesignDoc = function() {
    function DesignDoc(ddoc) {
      this.ddoc = ddoc;
    }
    DesignDoc.prototype.compile = function(funPath) {
      return new CouchFunction(this.ddoc, funPath);
    };
    DesignDoc.prototype.call = function(funPath, funArgs) {
      var _this;
      if (funArgs == null) {
        funArgs = [];
      }
      return (_this = this.compile(funPath)).call.apply(_this, funArgs);
    };
    return DesignDoc;
  }();
  CouchFunction = function() {
    function CouchFunction(ddoc, funPath) {
      var code, sandbox;
      this.ddoc = ddoc;
      this.funPath = funPath;
      this.log = [];
      this.fileName = "" + this.ddoc._id + "/" + this.funPath + ".js";
      code = "(" + (readPath(this.funPath, this.ddoc)) + ");";
      sandbox = createSandbox(this);
      try {
        this.fun = runInNewContext(code, sandbox, this.fileName);
      } catch (e) {

      }
      if (typeof this.fun !== 'function') {
        throw new NotAFunctionError(this.funPath, this.ddoc);
      }
    }
    CouchFunction.prototype.call = function() {
      var funArgs;
      funArgs = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return this.fun.apply(null, funArgs);
    };
    return CouchFunction;
  }();
  readPath = function(propPath, obj) {
    var getSubObject;
    getSubObject = function(o, prop) {
      if (typeof o === 'object' && prop in o) {
        return o[prop];
      } else {
        throw new MissingPropPathError(propPath, obj);
      }
    };
    return propPath.split('.').reduce(getSubObject, obj);
  };
  createSandbox = function(couchFun) {
    return {
      log: function(msg) {
        return couchFun.log.push(msg);
      },
      require: makeRequireFun(makeInitialRefStack(couchFun))
    };
  };
  makeInitialRefStack = function(couchFun) {
    var stack;
    stack = [couchFun.ddoc];
    couchFun.funPath.split('.').forEach(function(prop) {
      var currObj;
      currObj = stack[stack.length - 1];
      return stack.push(currObj[prop]);
    });
    stack.pop();
    return stack;
  };
  makeRequireFun = function(refStack) {
    return function(moduleID) {
      var code, nextRefStack, sandbox, _ref;
      sandbox = {
        exports: {}
      };
      try {
        _ref = findModule(moduleID, refStack), nextRefStack = _ref[0], code = _ref[1];
        sandbox.require = makeRequireFun(nextRefStack);
        runInNewContext(code, sandbox, moduleID);
      } catch (e) {
        throw "Design doc require() error: " + e.message;
      }
      return sandbox.exports;
    };
  };
  findModule = function(moduleID, refStack) {
    var code, current, next, part, _i, _len, _ref;
    _ref = moduleID.split('/');
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      part = _ref[_i];
      if (part === '..') {
        refStack.pop();
      } else if (part !== '.') {
        current = refStack[refStack.length - 1];
        next = current[part];
        if (next == null) {
          throw ("Couldn't find part: " + part)(moduleID);
        }
        refStack.push(next);
      }
    }
    code = refStack.pop();
    return [refStack, code];
  };
  exports.MissingPropPathError = MissingPropPathError = function() {
    function MissingPropPathError(funPath, ddoc) {
      this.funPath = funPath;
      this.ddoc = ddoc;
      this.message = "Property path '" + funPath + "' not found in design doc '" + this.ddoc._id + "'";
    }
    __extends(MissingPropPathError, Error);
    return MissingPropPathError;
  }();
  exports.NotAFunctionError = NotAFunctionError = function() {
    function NotAFunctionError(funPath, ddoc) {
      this.funPath = funPath;
      this.ddoc = ddoc;
      this.message = "Property path '" + funPath + "' in design doc '" + this.ddoc._id + "' does not evaluate to a function";
    }
    __extends(NotAFunctionError, Error);
    return NotAFunctionError;
  }();
}).call(this);
